{
   Copyright (C) 2010 MegaBrutal

   This unit is free software: you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This unit is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
}


unit NetRFC;

{$MODE DELPHI}

interface
uses SysUtils, Classes, SocketUtils;

type

   PRFCReply = ^TRFCReply;
   TRFCReply = class
   private
      FNumericCode: word;
      FReplyText: TStringList;
   public
      constructor Create;
      destructor Destroy; override;
      function Count: integer;
      function GetLine(n: integer): string;
      function GetNumericCode: word;
      procedure Add(Line: string);
      procedure Clear;
      procedure SetNumericCode(Code: word);
      procedure SetReply(NumericCode: word; Text: shortstring);
      property NumericCode: word read GetNumericCode;
      property ReplyText: TStringList read FReplyText;
   end;

   function ReadCommand(Sock: socket; var Command: shortstring; var Prms: string): boolean;
   function ReadResponse(Sock: socket; Response: TRFCReply): boolean;
   function SendCommand(Sock: socket; Command: shortstring): boolean; overload;
   function SendCommand(Sock: socket; Command: shortstring; Prms: string): boolean; overload;
   function SendResponse(Sock: socket; Response: TRFCReply): boolean;


implementation

constructor TRFCReply.Create;
begin
   inherited Create;
   FNumericCode:= 0;
   FReplyText:= TStringList.Create;
   FReplyText.Clear;
end;

destructor TRFCReply.Destroy;
begin
   FReplyText.Free;
   inherited Destroy;
end;

function TRFCReply.Count: integer;
begin
   Count:= FReplyText.Count;
end;

function TRFCReply.GetLine(n: integer): string;
begin
   GetLine:= FReplyText.Strings[n];
end;

function TRFCReply.GetNumericCode: word;
begin
   GetNumericCode:= FNumericCode;
end;

procedure TRFCReply.Add(Line: string);
begin
   FReplyText.Add(Line);
end;

procedure TRFCReply.Clear;
begin
   SetNumericCode(0);
   FReplyText.Clear;
end;

procedure TRFCReply.SetNumericCode(Code: word);
begin
   FNumericCode:= Code;
end;

procedure TRFCReply.SetReply(NumericCode: word; Text: shortstring);
begin
   SetNumericCode(NumericCode);
   FReplyText.Clear;
   FReplyText.Add(Text);
end;


function ReadCommand(Sock: socket; var Command: shortstring; var Prms: string): boolean;
var Line: string; i: integer;
begin
   ReadCommand:= true;
   try
      if SockReadLn(Sock, Line) then begin
         i:= pos(#32, Line);
         if i > 0 then begin
            Command:= Copy(Line, 1, i - 1);
            Prms:= Copy(Line, i + 1, Length(Line) - i);
         end
         else begin
            Command:= Line;
            Prms:= '';
         end;
      end
      else ReadCommand:= false;
   except
      ReadCommand:= false;
   end;
end;

function ReadResponse(Sock: socket; Response: TRFCReply): boolean;
var Line: string; ReadOK: boolean;
begin
   ReadResponse:= true;
   try
      Response.Clear;
      repeat
         ReadOK:= SockReadLn(Sock, Line);
         if ReadOK then Response.Add(Copy(Line, 5, Length(Line) - 4));
      until (Line[4] = #32) or (not ReadOK);
      if ReadOK then Response.SetNumericCode(StrToInt(Copy(Line, 1, 3)));
      ReadResponse:= ReadOK;
   except
      ReadResponse:= false;
   end;
end;

function SendCommand(Sock: socket; Command: shortstring): boolean;
begin
   try
      SendCommand:= SockWriteLn(Sock, Command);
   except
      SendCommand:= false;
   end;
end;

function SendCommand(Sock: socket; Command: shortstring; Prms: string): boolean;
begin
   try
      SendCommand:= SockWriteLn(Sock, Command + #32 + Prms);
   except
      SendCommand:= false;
   end;
end;

function SendResponse(Sock: socket; Response: TRFCReply): boolean;
var c, i: integer;
begin
   try
      c:= Response.Count;
      if c > 0 then begin
         for i:= 0 to c - 2 do SockWriteLn(Sock, IntToStr(Response.GetNumericCode) + '-' + Response.GetLine(i));
         SendResponse:= SockWriteLn(Sock, IntToStr(Response.GetNumericCode) + #32 + Response.GetLine(c - 1));
      end;
   except
      SendResponse:= false;
   end;
end;


end.
